﻿using System;
using System.Security.Cryptography;
using System.IO;

/*
 *         Progetto: Cifratura/decifratura AES mediante FileStream
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 01/04/2017
 *  
 *      Descrizione: Cifratura e decifratura di file mediante algoritmo AES.
 */

namespace AESFile
{
    class Program
    {
        // Vettore di inizializzazione (salt) usato per derivare la chiave a partire
        // da una password testuale. 
        // Utilizzata in entrambe le fasi di cifratura e decifratura.

        static readonly byte[] IVPassword = new byte[8] { 01, 04, 20, 17, 19, 20, 00, 00 };

        static void CifraAES()
        {
            string plainFile, cipherFile;
            string pw;
            Console.Clear();
            do
            {
                Console.Write("Nome del file da cifrare: ");
                plainFile = Console.ReadLine().Trim();
            }
            while (plainFile.Length == 0);
            do
            {
                Console.Write("Nome del file di output: ");
                cipherFile = Console.ReadLine().Trim();
            }
            while (cipherFile.Length == 0);


            Console.Write("Password: ");
            pw = Console.ReadLine();

            // Genera una chiave AES di lunghezza apposita a partire
            // dalla password inserita dall'utente
            var pdb = new Rfc2898DeriveBytes(pw, IVPassword);
            byte[] k = pdb.GetBytes(32);        // Chiave AES da 32 byte => 256 bit
            byte[] IVAlgoritmo = pdb.GetBytes(16);  // IV da 16 byte (128 bit) per elaborare blocchi AES

            var csp = new AesCryptoServiceProvider();
            var enc = csp.CreateEncryptor(k, IVAlgoritmo);

            Console.WriteLine("Lunghezza della chiave: {0} bit", k.Length * 8);

            using (var istream = new FileStream(plainFile, FileMode.Open))
            using (var ostream = new FileStream(cipherFile, FileMode.Create))
            {

                using (var cstream = new CryptoStream(ostream, enc, CryptoStreamMode.Write))
                {
                    istream.CopyTo(cstream);
                }
            }

            Console.WriteLine();
            Console.WriteLine("Documento cifrato e memorizzato in {0}", cipherFile);
            Console.WriteLine();
            Console.Write("Premere un tasto per continuare...");

            Console.ReadKey();
        }


        static void DecifraAES()
        {
            string plainFile, cipherFile;
            string pw;
            Console.Clear();
            do
            {
                Console.Write("Nome del file da decifrare: ");
                cipherFile = Console.ReadLine().Trim();
            }
            while (cipherFile.Length == 0);
            do
            {
                Console.Write("Nome del file di output: ");
                plainFile = Console.ReadLine().Trim();
            }
            while (plainFile.Length == 0);


            Console.Write("Password: ");
            pw = Console.ReadLine();

            // Genera una chiave AES di lunghezza apposita a partire
            // dalla password inserita dall'utente
            var pdb = new Rfc2898DeriveBytes(pw, IVPassword);
            byte[] k = pdb.GetBytes(32);        // Chiave AES da 32 byte => 256 bit
            byte[] IVAlgoritmo = pdb.GetBytes(16);  // IV da 16 byte (128 bit) per elaborare blocchi AES

            var csp = new AesCryptoServiceProvider();

            var dec = csp.CreateDecryptor(k, IVAlgoritmo);

            Console.WriteLine("Lunghezza della chiave: {0} bit", k.Length * 8);

            using (var istream = new FileStream(cipherFile, FileMode.Open))
            using (var ostream = new FileStream(plainFile, FileMode.Create))
            {

                using (var cstream = new CryptoStream(ostream, dec, CryptoStreamMode.Write))
                {
                    istream.CopyTo(cstream);
                }
            }

            Console.WriteLine();
            Console.WriteLine("Documento decifrato e memorizzato in {0}", plainFile);
            Console.WriteLine();
            Console.Write("Premere un tasto per continuare...");

            Console.ReadKey();
        }

        static void Main(string[] args)
        {
            int scelta;

            do
            {
                Console.Clear();
                Console.WriteLine("Algoritmo di cifratura AES mediante FileStream");
                Console.WriteLine();
                Console.WriteLine("1. Cifra file");
                Console.WriteLine("2. Decifra file");
                Console.WriteLine("0. Uscita");
                Console.WriteLine();
                Console.Write("Scelta: ");


                if (!Int32.TryParse(Console.ReadLine(), out scelta))
                    scelta = -1;

                try
                {
                    switch (scelta)
                    {
                        case 1:
                            CifraAES();
                            break;
                        case 2:
                            DecifraAES();
                            break;
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine();
                    Console.WriteLine(String.Format("Eccezione: {0}", ex.Message));
                    Console.WriteLine();
                    Console.Write("Premere un tasto per continuare...");
                    Console.ReadKey();
                }
            }
            while (scelta != 0);
        }
    }
}
