﻿using System;
using System.Text;
using System.Security.Cryptography;
using System.IO;

/*
 *         Progetto: Cifratura/decifratura AES mediante MemoryStream
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 01/04/2017
 *  
 *      Descrizione: Cifratura e decifratura di stringhe mediante algoritmo AES.
 */

namespace AESMemoria
{
    class Program
    {
        // Vettore di inizializzazione (salt) usato per derivare la chiave a partire
        // da una password testuale. 
        // Utilizzata in entrambe le fasi di cifratura e decifratura.

        static readonly byte[] IVPassword = new byte[8] { 01, 04, 20, 17, 19, 20, 00, 00 };

        static void CifraAES()
        {
            string plainText, cipherText;
            string pw;
            Console.Clear();
            Console.Write("Stringa Unicode da cifrare: ");
            plainText = Console.ReadLine();
            Console.Write("Password: ");
            pw = Console.ReadLine();

            // Genera una chiave AES di lunghezza apposita a partire
            // dalla password inserita dall'utente
            var pdb = new Rfc2898DeriveBytes(pw, IVPassword);
            byte[] k = pdb.GetBytes(32);        // Chiave AES da 32 byte => 256 bit
            byte[] IVAlgoritmo = pdb.GetBytes(16);  // IV da 16 byte (128 bit) per elaborare blocchi AES

            var csp = new AesCryptoServiceProvider();
            var enc = csp.CreateEncryptor(k, IVAlgoritmo);

            Console.WriteLine("Lunghezza della chiave: {0} bit", k.Length * 8);

            using (var istream = new MemoryStream(Encoding.UTF8.GetBytes(plainText)))
            using (var ostream = new MemoryStream())
            {

                using (var cstream = new CryptoStream(ostream, enc, CryptoStreamMode.Write))
                {
                    istream.CopyTo(cstream);
                }

                cipherText = Convert.ToBase64String(ostream.ToArray());
            }

            Console.WriteLine();
            Console.WriteLine("Messaggio cifrato (Base64): {0}", cipherText);
            Console.WriteLine();
            Console.Write("Premere un tasto per continuare...");

            Console.ReadKey();
        }


        static void DecifraAES()
        {
            string plainText, cipherText;
            string pw;
            Console.Clear();
            Console.Write("Stringa Base64 da decifrare: ");
            cipherText = Console.ReadLine();
            Console.Write("Password: ");
            pw = Console.ReadLine();

            // Genera una chiave AES di lunghezza apposita a partire
            // dalla password inserita dall'utente
            var pdb = new Rfc2898DeriveBytes(pw, IVPassword);
            byte[] k = pdb.GetBytes(32);        // Chiave AES da 32 byte => 256 bit
            byte[] IVAlgoritmo = pdb.GetBytes(16);  // IV da 16 byte (128 bit) per elaborare blocchi AES

            var csp = new AesCryptoServiceProvider();

            var dec = csp.CreateDecryptor(k, IVAlgoritmo);

            Console.WriteLine("Lunghezza della chiave: {0} bit", k.Length * 8);

            using (var istream = new MemoryStream(Convert.FromBase64String(cipherText)))
            using (var ostream = new MemoryStream())
            {

                using (var cstream = new CryptoStream(ostream, dec, CryptoStreamMode.Write))
                {
                    istream.CopyTo(cstream);
                }

                plainText = Encoding.UTF8.GetString(ostream.ToArray());
            }

            Console.WriteLine();
            Console.WriteLine("Messaggio Unicode decifrato: {0}", plainText);
            Console.WriteLine();
            Console.Write("Premere un tasto per continuare...");

            Console.ReadKey();

        }
        static void Main(string[] args)
        {
            int scelta;

            do
            {
                Console.Clear();
                Console.WriteLine("Algoritmo di cifratura AES mediante MemoryStream");
                Console.WriteLine();
                Console.WriteLine("1. Cifra");
                Console.WriteLine("2. Decifra");
                Console.WriteLine("0. Uscita");
                Console.WriteLine();
                Console.Write("Scelta: ");


                if (!Int32.TryParse(Console.ReadLine(), out scelta))
                    scelta = -1;

                try
                {
                    switch (scelta)
                    {
                        case 1:
                            CifraAES();
                            break;
                        case 2:
                            DecifraAES();
                            break;
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine();
                    Console.WriteLine(String.Format("Eccezione: {0}", ex.Message));
                    Console.WriteLine();
                    Console.Write("Premere un tasto per continuare...");
                    Console.ReadKey();
                }
            }
            while (scelta != 0);
        }
    }
}
